@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 # ldap-users.pl - Administrate users in ldap
 #
-# $Id: ldap-users.pl,v 1.90.2.50 2004/04/24 00:28:26 andreas Exp $
+# $Id: ldap-users.pl 2921 2005-02-18 23:18:43Z andreas $
 
 use Data::Dumper;
 use Net::LDAP;
@@ -23,14 +23,13 @@ my %g;    # this is "g" for global. but only global to this file,
 
 # Connect to LDAP server.
 sub ldap_connect {
-    my ( $server, $rootdn, $basedn, $minid, $maxid ) = @_;
+    my ( $server, $dn, $passwd, $basedn, $minid, $maxid ) = @_;
 
     my (@err, @dbg);
 
     $g{minid}         = $minid;
     $g{maxid}         = $maxid;
     $g{server}        = $server;
-    $g{rootdn}        = $rootdn;
     $g{basedn}        = $basedn;
     $g{ldap}          = undef;
     $g{err}           = \@err;
@@ -50,6 +49,17 @@ sub ldap_connect {
       . ldap_error_name( $result->code ) . " "
       . ldap_error_text( $result->code ) . " "
       if $result->code();
+
+    if ( defined $passwd ) {
+	debug ($passwd, $dn);
+	$result = $g{ldap}->bind( $dn, 'password' => $passwd );
+    
+	if ( $result->code ) {
+	    error_msg( "Could not bind to ldap server! "
+		       . ldap_error_name( $result->code ) . ": "
+		       . ldap_error_text( $result->code ) . "\n" );
+	}
+    }
 }
 
 sub ldap_check_capabilities {
@@ -268,15 +278,8 @@ sub ldap_search {
 }
 
 sub ldap_disable_user_login {
-    my ( $user_login, $rootpw ) = @_;
-    my $result = $g{ldap}->bind( $g{rootdn}, 'password' => $rootpw );
-    if ( $result->code ) {
-        error_msg( "Could not bind to ldap server! "
-            . ldap_error_name( $result->code ) . ": "
-            . ldap_error_text( $result->code ) . "\n" );
-        return undef;
-    }
-
+    my ( $user_login ) = @_;
+    my $result;
     my $base   = "ou=People," . $g{basedn};
     my $filter = "&(objectClass=posixAccount)(uid=$user_login)";
 
@@ -288,6 +291,10 @@ sub ldap_disable_user_login {
         return undef;
     }
 
+    if ( $config{sambasync} ){
+	system( "smbpasswd -d $user_login  >/dev/null 2>&1" );
+    }
+
     my $inactive_flag = $entry->get_value('shadowFlag');
     if ($inactive_flag) {
         debug_msg( "User $user_login is allready disabled by flag:"
@@ -351,15 +358,8 @@ sub ldap_disable_user_login {
 }
 
 sub ldap_enable_user_login {
-    my ( $user_login, $rootpw ) = @_;
-    my $result = $g{ldap}->bind( $g{rootdn}, 'password' => $rootpw );
-    if ( $result->code ) {
-        error_msg( "Could not bind to ldap server! "
-            . ldap_error_name( $result->code ) . ": "
-            . ldap_error_text( $result->code ) . "\n" );
-        return undef;
-    }
-
+    my ( $user_login ) = @_;
+    my $result;
     my $base   = "ou=People," . $g{basedn};
     my $filter = "(&(objectClass=posixAccount)(uid=$user_login))";
 
@@ -372,6 +372,10 @@ sub ldap_enable_user_login {
         return undef;
     }
 
+    if ( $config{sambasync} ){
+	system( "smbpasswd -e $user_login  >/dev/null 2>&1" );
+    }
+
     my $inactive_flag = $entry->get_value('shadowFlag');
     unless ($inactive_flag) {
         debug_msg( "User $user_login is allready ldap-enabled:"
@@ -655,6 +659,7 @@ sub ldap_get_not_member_grouplist {
     return @{$slice_ref};
 }
 
+# takes gidNumber, returns list of complete user entries
 sub ldap_get_member_userlist {
     my ($gidNumber) = @_;
 
@@ -673,29 +678,6 @@ sub ldap_get_member_userlist {
     return @list;
 }
 
-# Exemple ageGroup structure:
-# dn: cn=year2009,ou=Group,dc=skole
-# cn: exampleAgeGroup
-# memberUid: bernd     this is the user's uid
-# memberUid: kalle       ... teachers are in several age groups
-# ageGroup: mathe2     this is the group's cn
-# ageGroup: physics2     ... these classes are suitable for students that age
-
-# A Generic AgeGroup "GeAGroup" would contain both all students 
-# and all classes applicable for all students, regardless their age.
-
-# between schoolyears pupils and teachers are removed 
-# from their classes and moved to their new classes:
-# 1) the ageGroup attributes (reflecting the groups/classes) 
-#    are moved down to the following ageGroup
-# 2) the users are removed from those moved groups 
-#    and added to similar groups, which names index is increased by one,
-#    if those exist.
-# (this is done in step 1) those groups are then added to the ageGroup group
-#
-# one could search for missing groups in a step early in the 
-# process and offer to create those groups/abort.
-
 # get all users who are not yet members in a group
 sub ldap_get_not_member_userlist {
     my ($gidNumber) = @_;
@@ -712,59 +694,6 @@ sub ldap_get_not_member_userlist {
     my @black_list = $entry->get_value("memberUid");
     my $cn         = $entry->get_value("cn");
 
-=head1
-    # find the age groups this group is part of.
-    $base = "ou=Group," . $g{basedn};
-    my $filter_fix = "&(groupType=age_group)(ageGroup=$cn)";
-    my $filter_var = "cn=*";
-    my ( $slice_ref, $filters_applied ) =
-      get_slice( $base, $filter_fix, $filter_var );
-    if ( -1 == $#$slice_ref ) {
-
-        # this group is not part of an ageGroup, 
-        # use the worst case (generic ageGroup) instead.
-        my $filter = "(&(objectClass=lisGroup)(cn=genagegp))";
-        my $mesg   = $g{ldap}->search( base => $base, filter => $filter );
-        my $entry  = $mesg->pop_entry;
-        $slice_ref = [$entry];
-    }
-
-    # from those age groups use all users
-    # put them in a hash, for faster access and to "uniq" them
-    my %all_uids;
-    for my $group (@$slice_ref) {
-        for my $uid ( $group->get_value("memberUid") ) {
-            $all_uids{$uid} = 1;
-        }
-    }
-
-    # remove the users in the blacklist
-    for my $uid (@black_list) {
-        delete $all_uids{$uid};    # if ( $all_uids{$uid} );
-    }
-
-    my $filter_fix_append;
-
-    # we now have the uids of the wanted users ready, 
-    # but we need to get their complete entries
-    # therefor we get all user entries   
-    for my $uid ( keys %all_uids ) {
-        $filter_fix_append .= "(uid=$uid)";
-    }
-    $base = "ou=People," . $g{basedn};
-    if ($filter_fix_append) {
-        $filter_fix = "&(objectClass=posixAccount)(|$filter_fix_append)";
-    }
-    else {
-        return undef;
-
-        #$filter_fix = "&(objectClass=posixAccount)";
-    }
-    ( $slice_ref, $filters_applied ) =
-      get_slice( $base, $filter_fix, $filter_var );
-    return @{$slice_ref};
-=cut
-
     #make blacklist a hash, for faster access
     my (%blacklist, $blacklist_filter);
     for my $uid (@black_list) {
@@ -784,16 +713,6 @@ sub ldap_get_not_member_userlist {
   
     return @$slice_ref;
 
-#    my @ret_list;
-#    # remove the users in the blacklist
-#    for my $user (@$slice_ref) {
-#        my $uid = $user->get_value("uid");  
-#        unless ( $blacklist{$uid} ) {
-#            push @ret_list, $user;
-#        }
-#    }
-#    
-#    return @ret_list;
 }
 
 # Get all groups a user $uid is not member of, given a domain $basedn.
@@ -985,21 +904,13 @@ sub ldap_create_entry_nextid {
 # function fetched from http://www.ccie.org.uk/resources/newuid.html 
 # function somewhat rewritten to suit our needs
 sub ldap_get_newid {
-    my ($rootpw) = @_;
+#    my ($rootpw) = @_;
     my $loop = 2;
     my ( $result, $object, $newid );
     my $minid = $g{minid};
     my $maxid = $g{maxid};
     my $base  = "ou=variables," . $g{basedn};
 
-    $result = $g{ldap}->bind( $g{rootdn}, 'password' => $rootpw );
-    if ( $result->code ) {
-        error_msg( "Could not bind to ldap server! "
-            . ldap_error_name( $result->code ) . ": "
-            . ldap_error_text( $result->code ) . "\n" );
-        return undef;
-    }
-
     while ($loop) {
         $result = $g{ldap}->search(
             base   => $base,
@@ -1089,13 +1000,18 @@ sub ldap_get_newid {
 # Add a user to LDAP.
 sub ldap_add_user {
     my (
-        $cn,        $uid,    $userpw,  $uidNumber,
-        $gidNumber, $rootpw, $homedir, $maildir
+        $cn,        $uid,       $userpw,
+        $uidNumber, $gidNumber, 
+        $homedir,   $maildir,   $userpw_crypt
       )
       = @_;
 
+    # if there is no given crypt-hash generate it
+    unless ( $userpw_crypt ) {
+        $userpw_crypt = gen_crypt($userpw);
+    }
+
     debug_msg("Trying to add this users: uidNumber:$uidNumber  uid:$uid!\n");
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );    #XXX no error check!
     my $entry     = Net::LDAP::Entry->new();
     my $new_entry = "uid=$uid,ou=People," . $g{basedn};
     $entry->dn($new_entry);
@@ -1107,18 +1023,42 @@ sub ldap_add_user {
         gidNumber     => $gidNumber,
         homeDirectory => $homedir,
         mailMessageStore => $maildir,
-        userPassword     => "{crypt}" . gen_crypt($userpw),
+        userPassword     => "{crypt}" . $userpw_crypt,
         loginShell       => "/bin/bash",
     );
 
     return ldap_update($entry);
 }
 
+# Add a machine to LDAP.
+sub ldap_add_machine {
+    my (
+        $cn,        $uid,  , $uidNumber,
+        $gidNumber 
+	)
+	= @_;
+    
+    debug_msg("Trying to add this machine: uidNumber:$uidNumber  uid:$uid!\n");
+    my $entry     = Net::LDAP::Entry->new();
+    my $new_entry = "uid=$uid,ou=Machines,ou=People," . $g{basedn};
+    $entry->dn($new_entry);
+    $entry->add(
+		objectclass   => [ 'posixAccount', 'top', 'account'],
+		cn            => $cn,
+		uid           => $uid,
+		uidNumber     => $uidNumber,
+		gidNumber     => $gidNumber,
+		homeDirectory => "/dev/null",
+		loginShell       => "/bin/false",
+		);
+    
+    return ldap_update($entry);;
+}
+
 # Change an attribute $attr to the value $value for the user $uid.
 sub ldap_mod_user {
-    my ( $uid, $rootpw, $attr, $value ) = @_;
+    my ( $uid, $attr, $value ) = @_;
 
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
     $g{ldap}->modify( "uid=$uid,ou=People," . $g{basedn},
         replace => { $attr => "$value" } );
     $g{modified} = 1;
@@ -1127,7 +1067,7 @@ sub ldap_mod_user {
 # Change many attributes/value pairs in the 
 #   hash-ref $changes_ref for the user $uid.
 sub ldap_modify_user {
-    my ( $uid, $changes_ref, $rootpw ) = @_;
+    my ( $uid, $changes_ref ) = @_;
 
     my %additions;    # this is for new fields in the ldap-entry
     my $user = ldap_get_user($uid);
@@ -1139,9 +1079,6 @@ sub ldap_modify_user {
             delete $changes_ref->{$key};
         }
     }
-    my $auth_msg = $g{ldap}->bind( $g{rootdn}, password => $rootpw );
-    return text("auth_failed") if $auth_msg->code();
-    
 
     my $mesg = $g{ldap}->modify(
         "uid=$uid,ou=People," . $g{basedn}, replace => $changes_ref,
@@ -1158,7 +1095,7 @@ sub ldap_modify_user {
 # Change attributes/value pairs in the 
 #   hash-ref $changes_ref for the group $gidNumber.
 sub ldap_modify_group {
-    my ( $cn, $changes_ref, $rootpw ) = @_;
+    my ( $cn, $changes_ref ) = @_;
 
     my %additions;    # this is for new fields in the ldap-entry
     my $group = ldap_get_group($cn);
@@ -1168,7 +1105,6 @@ sub ldap_modify_group {
             delete $changes_ref->{$key};
         }
     }
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
 
     if ( keys %additions ) {
         my $mesg =
@@ -1193,17 +1129,20 @@ sub ldap_modify_group {
 
 # Make a new group $gid, with number $gidNumber, to LDAP.
 sub ldap_add_group {
-    my ( $cn, $gidNumber, $rootpw, $type, 
-#         $age_groups_ref, 
+    my ( $cn, $gidNumber, $type, 
          $description ) = @_;
 
+    if( getgrnam( $cn ) ) {
+	error_msg("Group $cn exists allready. ");
+	return undef;
+    } 
+
     unless ($type)        { $type        = "dontcare" }
     unless ($description) { $description = "dontcare" }
-
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
+    
     my $entry = Net::LDAP::Entry->new();
     $entry->dn( "cn=$cn,ou=Group," . $g{basedn} );
-
+    
     if ( $type eq "authority_group" ) {
         $entry->add(
             objectclass => [ 'lisAclGroup', 'top', 'lisGroup' ],
@@ -1223,115 +1162,14 @@ sub ldap_add_group {
             description => $description,
         );
     }
-#    for my $age_group ( @{$age_groups_ref} ) {
-#        ldap_add_group_to_age_group( $cn, $age_group, $rootpw );
-#    }
     return ldap_update($entry);
 }
 
-sub ldap_add_group_to_age_group {
-    my ( $cn, $age_group, $rootpw ) = @_;
-
-    my $mesg = $g{ldap}->search(
-        base   => $g{basedn},
-        filter => "(cn=$age_group)"
-    );
-
-    my $group_entry = $mesg->pop_entry;
-    return "agegroupdoesntexist" unless ($group_entry);
-
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
-
-    my $group_type = $group_entry->get_value("grouptype");
-    if ( $group_type eq "age_group" ) {
-        $g{ldap}->modify(
-            "cn=$age_group,ou=Group," . $g{basedn},
-            add => { 'ageGroup' => "$cn" }
-        );
-	$g{modified} = 1;
-        return "groupaddedtoagegroup";
-    }
-    else {
-        return "targetgroupisnoagegroup";
-    }
-}
-
-sub ldap_update_age_groups {
-
-    # group cn wants to be member in all the 
-    # ageGroups given in $age_groups_ref
-    my ( $cn, $age_groups_ref, $rootpw ) = @_;
-
-    my %age_groups_target;
-    for (@$age_groups_ref) {
-        $age_groups_target{$_} = 1;
-    }
-
-    for my $entry ( ldap_find_age_groups($cn) ) {
-        my $cn_actual = $entry->get_value("cn");
-
-        # eliminate the groups which are correct allready.
-        if ( $age_groups_target{$cn_actual} ) {
-            delete $age_groups_target{$cn_actual};
-        }
-        else {
-
-            # delete the once that are not wanted anymore
-            ldap_del_group_from_age_group( $cn, $cn_actual, $rootpw );
-        }
-    }
-
-    # now we have the target groups left, 
-    # which need to be made member in the age groups
-    for my $cn_age_group ( keys %age_groups_target ) {
-        ldap_add_group_to_age_group( $cn, $cn_age_group, $rootpw );
-    }
-}
-
-sub ldap_find_age_groups {
-    my ($cn) = @_;
-
-    my $base       = $g{basedn};
-    my $filter_fix = "&(objectClass=lisGroup)(ageGroup=$cn)";
-    my $filter_var = "cn=*";
-    my ( $slice_ref, $filters_applied ) =
-      get_slice( $base, $filter_fix, $filter_var );
-
-    return @{$slice_ref};
-}
-
-sub ldap_del_group_from_age_group {
-    my ( $cn, $age_group, $rootpw ) = @_;
-
-    my $mesg = $g{ldap}->search(
-        base   => $g{basedn},
-        filter => "(&(cn=$age_group)(ageGroup=$cn))"
-    );
-
-    my $group_entry = $mesg->pop_entry;
-    return "no such agegroup with that membergroup exists"
-      unless ($group_entry);
-
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
-
-    my $group_type = $group_entry->get_value("grouptype");
-    if ( $group_type eq "age_group" ) {
-        $g{ldap}->modify(
-            "cn=$age_group,ou=Group," . $g{basedn},
-            delete => { 'ageGroup' => "$cn" }
-        );
-	$g{modified} = 1;
-        return "groupdeletedfromagegroup";
-    }
-    else {
-        return "targetgroupisnoagegroup";
-    }
-}
 
 # Add a user $uid to group $gid.
 #strings returned correspond to strings in the lang/en file!
 sub ldap_add_user_to_group {
-    my ( $uid, $gid, $rootpw ) = @_;
+    my ( $uid, $gid ) = @_;
 
     my $mesg = $g{ldap}->search(
         base   => $g{basedn},
@@ -1344,7 +1182,6 @@ sub ldap_add_user_to_group {
     );
     my $group_entry = $mesg->pop_entry;
     return "groupdoesntexist" unless ($group_entry);
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
     my $group_type = $group_entry->get_value("grouptype");
 
     if ( $group_type ne "authority_group" ) {
@@ -1368,7 +1205,7 @@ sub ldap_add_user_to_group {
 
 # Delete a user $uid from group $gid. 
 sub ldap_del_user_from_group {
-    my ( $uid, $gid, $rootpw ) = @_;
+    my ( $uid, $gid ) = @_;
 
     my $mesg = $g{ldap}->search(
         base   => "ou=Group," . $g{basedn},
@@ -1376,7 +1213,7 @@ sub ldap_del_user_from_group {
     );
     my $group_entry = $mesg->pop_entry;
     return "(not a member)" unless ($group_entry);
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
+
     my $group_type = $group_entry->get_value("grouptype");
     if ( $group_type ne "authority_group" ) {
         $mesg = $g{ldap}->modify(
@@ -1399,25 +1236,30 @@ sub ldap_del_user_from_group {
 
 # Delete a user $uid.
 sub ldap_delete_user {
-    my ( $uid, $rootpw ) = @_;
+    my ( $uid ) = @_;
+    
+    $g{modified} = 1;
+    my $result = $g{ldap}->delete( "uid=$uid,ou=People,".$g{basedn} );
+    return $result->code;    
+}
+
+# Move a user $uid to the attic.
+sub ldap_remove_user {
+    my ( $uid ) = @_;
     
     $g{modified} = 1;
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
     my $result = $g{ldap}->moddn( "uid=$uid,ou=People,".$g{basedn},
                                   newrdn => "uid=$uid",
                                   deleteoldrdn => 1,
                                   newsuperior => "ou=Attic,".$g{basedn},
                                   );
-    my $output .= ldap_error_name( $result->code ) . ": "
-      . ldap_error_text( $result->code );
     return $result->code;    
 }
 
 # Delete a group $gid.
 sub ldap_delete_group {
-    my ( $cn, $rootpw ) = @_;
+    my ( $cn ) = @_;
     $g{modified} = 1;
-    $g{ldap}->bind( $g{rootdn}, password => $rootpw );
     my $msg = $g{ldap}->delete( "cn=$cn,ou=Group," . $g{basedn} );
     return $msg->code;
 }
@@ -1426,7 +1268,7 @@ sub ldap_delete_group {
 sub print_ldap_error {
     my ( $errormsg, $result ) = @_;
     my $output = $errormsg;
-    $output .= $text{'error'} . ": \n";
+    $output .= text('error') . ": \n";
     $output .= ldap_error_name( $result->code ) . ": "
       . ldap_error_text( $result->code );
     return $output;
@@ -1449,34 +1291,6 @@ sub ldap_update {
     return $ret;
 }
 
-# Encrypt a password.
-sub gen_crypt {
-    my ($plain) = @_;
-    my $salt;
-
-    # Get a truly random salt.
-    open FILE, "</dev/urandom";
-    sysread FILE, $salt, 2;
-    close FILE;
-
-    # If crypt needs salt to be a set of usable characters, uncomment this.
-    #   srand($salt);
-    #   $salt = chr(int(rand(97)) + 25).chr(int(rand(97)) + 25);
-    # Encrypt.
-    return crypt( $plain, $salt );
-
-}
-
-sub make_unique_username {
-    my ( $firstname, $lastname, $prefered_login ) = @_;
-    my @names = user_suggest_unames( $firstname, $lastname, $prefered_login );
-    unless ( @names > 0 ) {
-        error_msg("Found no suitable unames from '$firstname $lastname'.\n");
-        return undef;
-    }
-    return $names[0];
-}
-
 sub make_cn {
     my ( $first_name, $last_name ) = @_;
 
@@ -1485,208 +1299,6 @@ sub make_cn {
     return $cn;
 }
 
-sub conv_name {
-    my ( $s, $alt ) = @_;
-
-
-    $s =~ s/[]/ae/g;    # Different mapping of  and ,
-    $s =~ s/[]/aa/g;    # used when creating usernames.
-
-
-    # Vask bort ymse sre tegn
-    $s =~ tr
-        ({[}]|\\)
-        (AOAaooaAAAAACEEEEIIIINOOOOOUUUUYaaaaaceeeeiiiinooooouuuuyaAaAooO);
-    $s =~ s//Dh/g;
-    $s =~ s//dh/g;
-    $s =~ s//Th/g;
-    $s =~ s//th/g;
-    $s =~ s//ss/g;
-    $s =~ s/[\200-\377]/x/g;
-    $s =~ tr/A-Z/a-z/;
-    $s =~ s/[^a-zA-Z0-9 ]//g;
-    return $s;
-}
-
-sub user_suggest_unames {
-    my ( $fname, $lname, $prefered_login ) = @_;
-    my $initial;
-    my $firstinit;
-    my @potuname = ();
-    my $goal     = 1;    # minimum number of alternatives to find.
-
-    # to make this work with cn as input.
-    $lname = ( split ( / /, $fname ) )[-1] unless $lname;
-
-    push ( @potuname, $prefered_login ) 
-	if ( $prefered_login and validate_new_uname($prefered_login) );
-    my $complete_name = &conv_name( "$fname $lname", 1 );
-
-    # Remember just the first initials.
-
-    if ( $complete_name =~ /^(.*)[ -]+(\S+)\s+(\S+)$/ ) {
-
-        # at least three names
-        $firstinit = $1;
-        $firstinit =~ s/([- ])(\S)[^- ]*/$1$2/g;
-        $firstinit =~ s/^(\S).*?($|[- ])/$1/;
-        $firstinit =~ s/[- ]//g;
-    }
-
-    # Remove hyphens.  People called "Geir-Ove Johnsen Hansen" generally
-    # prefer "geirove" to just "geir".
-
-    $complete_name =~ s/-//g;
-
-    $complete_name =~ /^(\S+)?(.*\s+(\S)\S*)?\s+(\S+)?$/;
-    $fname = substr( $1, 0, 8 );
-    $initial = $3;
-    $lname   = substr( $4, 0, 8 );
-
-    # print "DEBUG: split into '$fname' '$initial' '$lname'\n";
-
-    $lname = $fname unless $lname;    # Sane behaviour if only one name.
-
-    # First try the obvious, first and last name.  Should be
-    # restricted to interactively created accounts (== usually
-    # non-student).
-
-    if (0) {
-        push ( @potuname, $fname ) if validate_new_uname($fname);
-        push ( @potuname, $lname ) if validate_new_uname($lname);
-    }
-
-    # For people with many names, we prefer to use all initials:
-    # Example:  Geir-Ove Johnsen Hansen
-    #           ffff fff i       llllll
-    # Here, firstinit is "GO" and initial is "J".
-    #
-    # gohansen gojhanse gohanse gojhans ... gojh goh
-    # ssllllll ssilllll sslllll ssillll     ssil ssl
-    #
-    # ("ss" means firstinit, "i" means initial, "l" means last name)
-
-    my ( $i, $j, $try );
-
-    if ( $firstinit && length($firstinit) > 1 ) {
-        $i = length($firstinit);
-        my $llen = length($lname);
-        $llen = 8 - $i if ( $llen > 8 - $i );
-        for ( $j = $llen ; $j > 0 ; $j-- ) {
-            $try = $firstinit . substr( $lname, 0, $j );
-            push ( @potuname, $try ) if validate_new_uname($try);
-            if ( $j > 1 && $initial ) {
-                $try = $firstinit . $initial . substr( $lname, 0, $j - 1 );
-                push ( @potuname, $try ) if validate_new_uname($try);
-            }
-            last if @potuname >= $goal;
-        }
-    }
-
-    # Now try different substrings from first and last name.
-    #
-    # geiroveh,
-    # fffffffl
-    # geirovh geirovha geirovjh,
-    # ffffffl ffffffll ffffffil
-    # geiroh geirojh geiroha geirojha geirohan,
-    # fffffl fffffil fffffll fffffill ffffflll
-    # geirh geirjh geirha geirjha geirhan geirjhan geirhans
-    # ffffl ffffil ffffll ffffill fffflll ffffilll ffffllll
-    # ...
-    # gjh gh gjha gha gjhan ghan ... gjhansen ghansen
-    # fil fl fill fll filll flll     fillllll fllllll
-
-    my $flen = length($fname);
-
-    $flen = 7 if $flen > 7;
-    for ( $i = $flen ; $i > 0 ; $i-- ) {
-        my $llim = length($lname);
-        $llim = 8 - $i if ( $llim > 8 - $i );
-        for ( $j = 1 ; $j <= $llim ; $j++ ) {
-            if ($initial) {
-
-                # Is there room for an initial?
-                if ( $j == $llim && $i + $llim < 8 ) {
-                    $try =
-                      substr( $fname, 0, $i ) . $initial
-                      . substr( $lname, 0, $j );
-                    push ( @potuname, $try ) if validate_new_uname($try);
-                }
-
-                # Is there room for an initial if we chop a letter off
-                # last name?
-                if ( $j > 1 ) {
-                    $try =
-                      substr( $fname, 0, $i ) . $initial
-                      . substr( $lname, 0, $j - 1 );
-                    push ( @potuname, $try ) if validate_new_uname($try);
-                }
-            }
-            $try = substr( $fname, 0, $i ) . substr( $lname, 0, $j );
-            push ( @potuname, $try ) if validate_new_uname($try);
-        }
-        last if @potuname >= $goal;
-    }
-
-    # Absolutely last ditch effort:  geirov1, geirov2 etc.
-
-    $i = 1;
-    $flen = 6 if $flen > 6;
-    while ( @potuname < $goal && $i < 100 ) {
-        $try = substr( $fname, 0, $flen ) . $i++;
-        push ( @potuname, $try ) if validate_new_uname($try);
-    }
-
-    return @potuname;
-}
-
-sub validate_new_uname {
-    my ($uname) = @_;
-
-    # We only allow numbers and lowercase letters
-    if ( $uname =~ m/[^a-z0-9]/ ) {
-        error_msg(  text( "login_illegal_chars", $uname ) );
-        return undef;
-    }
-    elsif ( length($uname) <= 1 ) {
-        error_msg(  text( "login_too_short" ) );
-        return undef;
-
-        #    } elsif (gethostbyname($uname)) {
-        #       debug_msg("Username `$uname' in use as hostname");
-        #       return undef;
-    }
-    elsif ( ldap_get_user($uname) ) {
-        error_msg( text( "login_allready_used", $uname ) );
-        return undef;
-    }
-    elsif ( ldap_get_old_user($uname) ) {
-        error_msg( text ( "login_was_used", $uname ) );
-        return undef;
-    }
-    elsif ( getpwnam($uname) || grep { $_ eq $uname; }
-
-        qw(root bin daemon adm lp sync shutdown halt mail nobody
-        nobodyv ftp uucp uucpa auth cron core tcb ris wnn sys
-        nuucp hpdb opc_op predmail smbnull sysadm cmwlogin diag
-        auditor dbadmin sgiweb rfindd ezsetup demos outofbox
-        guest 4dgifts noaccess rpcuser)
-      )
-    {
-        error_msg("'$uname' is a system user.\n");
-        return undef;
-    }
-    elsif ( getpwnam($uname) || grep { $_ eq $uname; }
-
-        qw(shit fuck dimwit whore fuckwit)
-      )    # to be continued...
-    {
-        error_msg("'$uname' is a dirty word.\n");
-        return undef;
-    }
-    return 1;
-}
 
 sub error_msg {
     my ($mesg) = @_;
@@ -1716,20 +1328,6 @@ sub ldap_dbg_output {
     return $ret;
 }
 
-# this function does not at all belong in the ldap module. 
-# needs to go elsewhere later.
-sub read_adduser_config {
-    my $config_file = "/etc/adduser.conf";
-    read_file( $config_file, \%config );    # reuse a function from web-lib.pl
-         #now the config data is in the config hash. (c:
-
-}
-
-sub read_miniserv_config {
-    my $config_file = "/etc/webmin/miniserv.conf";
-    read_file( $config_file, \%config );    # reuse a function from web-lib.pl
-}
-
 1;
 
 __END__
